def load_cms_fingerprints():
    """Load CMS fingerprints from the built-in dictionary."""
    return {
        'wordpress': {
            'headers': ['x-powered-by: php'],
            'meta_tags': ['generator', 'wordpress'],
            'paths': ['/wp-login.php', '/wp-admin/', '/wp-content/', '/wp-includes/'],
            'html_patterns': ['wp-content', 'wp-includes', 'wp-image'],
            'js_patterns': ['wp-emoji', 'wp-embed', 'wp-'],
            'cookies': ['wordpress', 'wp-settings', 'wp_'],
        },
        'joomla': {
            'headers': ['x-content-encoded-by: joomla'],
            'meta_tags': ['generator', 'joomla'],
            'paths': ['/administrator/', '/components/', '/modules/', '/templates/'],
            'html_patterns': ['joomla', 'com_content', 'com_users'],
            'js_patterns': ['joomla', 'mootools'],
            'cookies': ['jfcookie', 'joomla_user_state'],
        },
        'drupal': {
            'headers': ['x-generator: drupal'],
            'meta_tags': ['generator', 'drupal'],
            'paths': ['/sites/default/', '/sites/all/', '/core/'],
            'html_patterns': ['drupal-', 'sites/all', 'sites/default'],
            'js_patterns': ['drupal', 'drupalSettings'],
            'cookies': ['drupal', 'Drupal'],
        },
        'magento': {
            'headers': ['x-magento-'],
            'meta_tags': ['msvalidate'],
            'paths': ['/skin/', '/media/', '/frontend/', '/adminhtml/'],
            'html_patterns': ['magento', 'Mage.Cookies', 'magentostore'],
            'js_patterns': ['magento', 'Mage'],
            'cookies': ['frontend=', 'adminhtml'],
        },
        'shopify': {
            'headers': ['x-shopify', 'x-shopid'],
            'meta_tags': ['shopify-checkout', 'shopify-digital-wallet'],
            'paths': ['/cdn.shopify.com', '/shopify-', '/s/files/'],
            'html_patterns': ['shopify', 'cdn.shopify.com', 'asset_url'],
            'js_patterns': ['shopify', 'Shopify.'],
            'cookies': ['_shopify', '_landing_page'],
        },
        'wix': {
            'headers': ['x-wix-request-id'],
            'meta_tags': ['generator', 'wix'],
            'paths': ['/wix-', '/_wix/'],
            'html_patterns': ['wix-', 'wixcode', 'wixsite'],
            'js_patterns': ['wix', 'wixBiSession'],
            'cookies': ['wixSession', 'wixLanguage'],
        },
        'squarespace': {
            'headers': ['x-servedby: squarespace'],
            'meta_tags': ['generator', 'squarespace'],
            'paths': ['/static.squarespace.com', '/assets/'],
            'html_patterns': ['squarespace', 'data-content-field'],
            'js_patterns': ['squarespace', 'Static.SQUARESPACE_CONTEXT'],
            'cookies': ['ss_cid', 'ss_cpvisit'],
        },
        'typo3': {
            'headers': ['x-typo3-', 'typo3-reverse-proxy'],
            'meta_tags': ['generator', 'typo3'],
            'paths': ['/typo3/', '/typo3conf/', '/typo3temp/'],
            'html_patterns': ['typo3', 't3-js-', 'TYPO3'],
            'js_patterns': ['TYPO3', 'typo3'],
            'cookies': ['typo3', 'be_typo_user'],
        },
        'ghost': {
            'headers': ['x-ghost-', 'x-powered-by: ghost'],
            'meta_tags': ['generator', 'ghost'],
            'paths': ['/ghost/', '/assets/ghost/'],
            'html_patterns': ['ghost', 'ghost-theme'],
            'js_patterns': ['ghost', 'Ghost'],
            'cookies': ['ghost-admin', 'ghost-token'],
        },
        'bigcommerce': {
            'headers': ['x-bc-'],
            'meta_tags': ['generator', 'bigcommerce'],
            'paths': ['/assets/bc-', '/bc-sf-filter/'],
            'html_patterns': ['bigcommerce', 'bc-product-'],
            'js_patterns': ['bigcommerce', 'BCData'],
            'cookies': ['bigcommerce_', 'fornax_'],
        },
        'opencart': {
            'headers': [],
            'meta_tags': ['generator', 'opencart'],
            'paths': ['/catalog/', '/admin/', '/system/storage/'],
            'html_patterns': ['opencart', 'route=product'],
            'js_patterns': ['opencart', 'cart.add'],
            'cookies': ['OCSESSID', 'currency'],
        },
        'prestashop': {
            'headers': ['powered-by: prestashop'],
            'meta_tags': ['generator', 'prestashop'],
            'paths': ['/modules/', '/themes/', '/img/p/'],
            'html_patterns': ['prestashop', 'presta-', '/modules/'],
            'js_patterns': ['prestashop', 'prestashop-modules'],
            'cookies': ['PrestaShop-', 'PSSESSID'],
        },
        'django': {
            'headers': ['csrftoken', 'django'],
            'meta_tags': [],
            'paths': ['/static/', '/media/'],
            'html_patterns': ['csrfmiddlewaretoken', 'django'],
            'js_patterns': ['django', 'csrftoken'],
            'cookies': ['csrftoken', 'django_'],
        },
        'laravel': {
            'headers': ['laravel_session'],
            'meta_tags': [],
            'paths': ['/vendor/laravel/', '/storage/'],
            'html_patterns': ['csrf-token', 'laravel'],
            'js_patterns': ['laravel', 'window.Laravel'],
            'cookies': ['laravel_session', 'XSRF-TOKEN'],
        },
        'ruby_on_rails': {
            'headers': ['powered-by: phusion passenger'],
            'meta_tags': ['csrf-param'],
            'paths': ['/assets/', '/rails/'],
            'html_patterns': ['data-method="delete"', 'csrf-token'],
            'js_patterns': ['rails', 'Rails.'],
            'cookies': ['_session_id', '_rails_'],
        }
    }